var gulp = require("gulp");
  var _ = require('underscore');

module.exports = {
    /**
     * This represents a task
     */
    newTask: function() {
        return new TaskBuilder();
    }
};

var TaskBuilder = function() {
    this._name = null;
    this._src = null;
    this._dest = null;
    this._prerequisiteTasks = [];
    this._processFunction = function(gulpTask) {
        return gulpTask;
    };

    this._includeWatchers = false;
};

TaskBuilder.prototype.name = function(name) {
    this._name = name;
    return this;
};

TaskBuilder.prototype.src = function(src) {
    this._src = src;
    return this;
};

TaskBuilder.prototype.dest = function(dest) {
    this._dest = dest;
    return this;
};

TaskBuilder.prototype.prerequisiteTask = function(task) {
    this._prerequisiteTasks.push(task);
    return this;
};

TaskBuilder.prototype.process = function(processFunction) {
    this._processFunction = processFunction;
    return this;
};

TaskBuilder.prototype.includeWatchers = function(includeWatchers) {
    this._includeWatchers = includeWatchers;
    return this;
};

TaskBuilder.prototype.build = function() {
    var self = this;

    var prerequisiteTasks = _.pluck(this._prerequisiteTasks, 'taskName');

    if (this._includeWatchers) {
        gulp.task(this._name + '-watch', function() {
            self._prerequisiteTasks.forEach(function (task) {
                console.log("Watching files: ", task.src, " and will do '" + task.taskName + "' if they change");
                gulp.watch(task.src, [task.taskName]);
            });
        });

        prerequisiteTasks.push(this._name + '-watch');
    }


    gulp.task(this._name, prerequisiteTasks, function() {
        if (self._src) {
            var src = gulp.src(self._src);

            if (self._processFunction) {
                var processedSrc = self._processFunction(src);

                if (self._dest) {
                    return processedSrc.pipe(gulp.dest(self._dest))
                }
            }
        }
    });

    return {
        taskName: self._name,
        src: self._src
    };
};